@@ -1,4 +1,4 @@
-# Copyright 2015-2017 Red Hat, Inc.
+# Copyright 2015-2018 Red Hat, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,7 +19,9 @@ import sys
 
 from openstack import config as os_config
 
+from metalsmith import _format
 from metalsmith import _provisioner
+from metalsmith import _utils
 
 
 LOG = logging.getLogger(__name__)
@@ -36,7 +38,9 @@ class NICAction(argparse.Action):
         setattr(namespace, self.dest, nics)
 
 
-def _do_deploy(api, args, wait=None):
+def _do_deploy(api, args, formatter):
+    wait = None if args.no_wait else args.wait
+
     capabilities = dict(item.split('=', 1) for item in args.capability)
     if args.ssh_public_key:
         with open(args.ssh_public_key) as fp:
@@ -44,18 +48,29 @@ def _do_deploy(api, args, wait=None):
     else:
         ssh_keys = []
 
+    if args.hostname and not _utils.is_hostname_safe(args.hostname):
+        raise RuntimeError("%s cannot be used as a hostname" % args.hostname)
+
     node = api.reserve_node(args.resource_class, capabilities=capabilities)
-    api.provision_node(node,
-                       image_ref=args.image,
-                       nics=args.nics,
-                       root_disk_size=args.root_disk_size,
-                       ssh_keys=ssh_keys,
-                       netboot=args.netboot,
-                       wait=wait)
+    instance = api.provision_node(node,
+                                  image_ref=args.image,
+                                  nics=args.nics,
+                                  root_disk_size=args.root_disk_size,
+                                  ssh_keys=ssh_keys,
+                                  hostname=args.hostname,
+                                  netboot=args.netboot,
+                                  wait=wait)
+    formatter.deploy(instance)
 
 
-def _do_undeploy(api, args, wait=None):
-    api.unprovision_node(args.node, wait=wait)
+def _do_undeploy(api, args, formatter):
+    node = api.unprovision_node(args.node, wait=args.wait)
+    formatter.undeploy(node)
+
+
+def _do_show(api, args, formatter):
+    instances = api.show_instances(args.instance)
+    formatter.show(instances)
 
 
 def _parse_args(args, config):
@@ -65,14 +80,16 @@ def _parse_args(args, config):
     verbosity.add_argument('-q', '--quiet', action='store_true',
                            help='output only errors')
     verbosity.add_argument('--debug', action='store_true',
-                           help='output more logging')
+                           help='output extensive logging')
+    verbosity.add_argument('-v', '--verbose', action='count', default=0,
+                           dest='verbosity',
+                           help='increase output verbosity, can be specified '
+                           'up to three times')
     parser.add_argument('--dry-run', action='store_true',
                         help='do not take any destructive actions')
-    wait = parser.add_mutually_exclusive_group()
-    wait.add_argument('--wait', type=int, default=1800,
-                      help='action timeout (in seconds)')
-    wait.add_argument('--no-wait', action='store_true',
-                      help='disable waiting for action to finish')
+    parser.add_argument('--format', choices=list(_format.FORMATS),
+                        default=_format.DEFAULT_FORMAT,
+                        help='output format')
 
     config.register_argparse_arguments(parser, sys.argv[1:])
 
@@ -80,6 +97,12 @@ def _parse_args(args, config):
 
     deploy = subparsers.add_parser('deploy')
     deploy.set_defaults(func=_do_deploy)
+    wait = deploy.add_mutually_exclusive_group()
+    wait.add_argument('--wait', type=int, default=1800,
+                      help='time (in seconds) to wait for node to become '
+                      'active')
+    wait.add_argument('--no-wait', action='store_true',
+                      help='disable waiting for deploy to finish')
     deploy.add_argument('--image', help='image to use (name or UUID)',
                         required=True)
     deploy.add_argument('--network', help='network to use (name or UUID)',
@@ -94,45 +117,73 @@ def _parse_args(args, config):
     deploy.add_argument('--capability', action='append', metavar='NAME=VALUE',
                         default=[], help='capabilities the nodes should have')
     deploy.add_argument('--ssh-public-key', help='SSH public key to load')
+    deploy.add_argument('--hostname', help='Host name to use, defaults to '
+                        'Node\'s name or UUID')
     deploy.add_argument('resource_class', help='node resource class to deploy')
 
     undeploy = subparsers.add_parser('undeploy')
     undeploy.set_defaults(func=_do_undeploy)
     undeploy.add_argument('node', help='node UUID')
+    undeploy.add_argument('--wait', type=int,
+                          help='time (in seconds) to wait for node to become '
+                          'available for deployment again')
+
+    show = subparsers.add_parser('show')
+    show.set_defaults(func=_do_show)
+    show.add_argument('instance', nargs='+', help='instance UUID(s)')
+
     return parser.parse_args(args)
 
 
-def _configure_logging(args):
-    log_fmt = ('%(asctime)s %(levelname)s %(name)s: %(message)s' if args.debug
-               else '[%(asctime)s] %(message)s')
-    if args.quiet:
-        level = logging.ERROR
-    elif args.debug:
-        level = logging.DEBUG
-    else:
-        level = logging.INFO
+_URLLIB3_LOGGER = 'urllib3.connectionpool'
 
-    logging.basicConfig(level=level, format=log_fmt)
-    if args.debug:
-        logging.getLogger('urllib3.connectionpool').setLevel(logging.INFO)
-    else:
-        logging.getLogger('urllib3.connectionpool').setLevel(logging.CRITICAL)
+
+def _configure_logging(args):
+    log_fmt = ('%(asctime)s %(levelname)s %(name)s: %(message)s'
+               if args.debug or args.verbosity
+               else '[%(asctime)s] %(message)s')
+
+    # Verbosity:
+    # 0 (the default) - warnings and errors
+    # 1 - info from metalsmith, warnings and errors from everything else
+    # 2 - debug from metalsmith, info from everything else
+    # 3 - the same as --debug
+    base_level = logging.WARNING
+    metalsmith_level = base_level
+    urllib_level = logging.CRITICAL
+
+    if args.quiet:
+        base_level = logging.CRITICAL
+        metalsmith_level = base_level
+    elif args.debug or args.verbosity > 2:
+        base_level = logging.DEBUG
+        metalsmith_level = base_level
+        urllib_level = logging.INFO
+    elif args.verbosity == 2:
+        base_level = logging.INFO
+        metalsmith_level = logging.DEBUG
+    elif args.verbosity == 1:
+        metalsmith_level = logging.INFO
+
+    logging.basicConfig(level=base_level, format=log_fmt)
+    logging.getLogger('metalsmith').setLevel(metalsmith_level)
+    logging.getLogger(_URLLIB3_LOGGER).setLevel(urllib_level)
 
 
 def main(args=sys.argv[1:]):
     config = os_config.OpenStackConfig()
     args = _parse_args(args, config)
     _configure_logging(args)
-    if args.no_wait:
-        wait = None
+    if args.quiet:
+        formatter = _format.NULL_FORMAT
     else:
-        wait = args.wait
+        formatter = _format.FORMATS[args.format]
 
     region = config.get_one(argparse=args)
     api = _provisioner.Provisioner(cloud_region=region, dry_run=args.dry_run)
 
     try:
-        args.func(api, args, wait=wait)
+        args.func(api, args, formatter)
     except Exception as exc:
         LOG.critical('%s', exc, exc_info=args.debug)
         sys.exit(1)
