@@ -13,15 +13,33 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import contextlib
 import logging
 
 from ironicclient import client as ir_client
 from openstack import connection
 import six
 
+from metalsmith import _utils
+
 
 LOG = logging.getLogger(__name__)
-REMOVE = object()
+NODE_FIELDS = ['name', 'uuid', 'instance_info', 'instance_uuid', 'maintenance',
+               'maintenance_reason', 'properties', 'provision_state', 'extra']
+HOSTNAME_FIELD = 'metalsmith_hostname'
+
+
+class _Remove(object):
+    """Indicator that a field should be removed."""
+
+    __slots__ = ()
+
+    def __repr__(self):
+        """Allow nicer logging."""
+        return '<REMOVE>'
+
+
+REMOVE = _Remove()
 
 
 class DictWithAttrs(dict):
@@ -40,6 +58,8 @@ class API(object):
     IRONIC_VERSION = '1'
     IRONIC_MICRO_VERSION = '1.28'
 
+    _node_list = None
+
     def __init__(self, session=None, cloud_region=None):
         if cloud_region is None:
             if session is None:
@@ -59,9 +79,22 @@ class API(object):
             self.IRONIC_VERSION, session=self.session,
             os_ironic_api_version=self.IRONIC_MICRO_VERSION)
 
+    def _nodes_for_lookup(self):
+        return self.list_nodes(maintenance=None,
+                               associated=None,
+                               provision_state=None,
+                               fields=['uuid', 'name', 'instance_info'])
+
     def attach_port_to_node(self, node, port_id):
         self.ironic.node.vif_attach(_node_id(node), port_id)
 
+    @contextlib.contextmanager
+    def cache_node_list_for_lookup(self):
+        if self._node_list is None:
+            self._node_list = self._nodes_for_lookup()
+        yield self._node_list
+        self._node_list = None
+
     def create_port(self, network_id, **kwargs):
         return self.connection.network.create_port(network_id=network_id,
                                                    admin_state_up=True,
@@ -74,6 +107,22 @@ class API(object):
     def detach_port_from_node(self, node, port_id):
         self.ironic.node.vif_detach(_node_id(node), port_id)
 
+    def find_node_by_hostname(self, hostname):
+        nodes = self._node_list or self._nodes_for_lookup()
+        existing = [n for n in nodes
+                    if n.instance_info.get(HOSTNAME_FIELD) == hostname]
+        if len(existing) > 1:
+            raise RuntimeError("More than one node found with hostname "
+                               "%(host)s: %(nodes)s" %
+                               {'host': hostname,
+                                'nodes': ', '.join(_utils.log_node(n)
+                                                   for n in existing)})
+        elif not existing:
+            return None
+        else:
+            # Fetch the complete node record
+            return self.get_node(existing[0].uuid, accept_hostname=False)
+
     def get_image_info(self, image_id):
         return self.connection.image.find_image(image_id,
                                                 ignore_missing=False)
@@ -82,9 +131,22 @@ class API(object):
         return self.connection.network.find_network(network_id,
                                                     ignore_missing=False)
 
-    def get_node(self, node):
+    def get_node(self, node, refresh=False, accept_hostname=False):
         if isinstance(node, six.string_types):
-            return self.ironic.node.get(node)
+            if accept_hostname and _utils.is_hostname_safe(node):
+                by_hostname = self.find_node_by_hostname(node)
+                if by_hostname is not None:
+                    return by_hostname
+
+            return self.ironic.node.get(node, fields=NODE_FIELDS)
+        elif hasattr(node, 'node'):
+            # Instance object
+            node = node.node
+        else:
+            node = node
+
+        if refresh:
+            return self.ironic.node.get(node.uuid, fields=NODE_FIELDS)
         else:
             return node
 
@@ -99,11 +161,13 @@ class API(object):
         return self.ironic.node.list_ports(_node_id(node), limit=0)
 
     def list_nodes(self, resource_class=None, maintenance=False,
-                   associated=False, provision_state='available', detail=True):
+                   associated=False, provision_state='available',
+                   fields=None):
         return self.ironic.node.list(limit=0, resource_class=resource_class,
                                      maintenance=maintenance,
-                                     associated=associated, detail=detail,
-                                     provision_state=provision_state)
+                                     associated=associated,
+                                     provision_state=provision_state,
+                                     fields=fields or NODE_FIELDS)
 
     def node_action(self, node, action, **kwargs):
         self.ironic.node.set_provision_state(_node_id(node), action, **kwargs)
